# frozen_string_literal: true

module Hermes
  # Dossier de candidature , gere le listing des pieces a fournir et le statut du dossier
  class HousingRequestFolder
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::LockableConcern
    include Hermes::Concerns::SopraXmlSender
    
    
    field :nud_status, type: String
    field :consultation_status, type: String
    field :application_status, type: String
    
    field :validity_limit, type: Date # date a partir de laquelle le dossier ne sera plus valide
    field :completion, type: Integer # pourcentage de completion du dossier
    field :file_upload_from_sne_complete, type: Boolean
    belongs_to :housing_request, class_name: 'Hermes::HousingRequest'
    has_and_belongs_to_many :required_documents, class_name: 'Hermes::RequiredDocument', inverse_of: nil
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    writer_auth_roles 'collaborateur entreprise', "collaborateur offre logement", 'superadministrateur'
    
    
    before_create :before_create_action
    before_save :before_save_action
    after_update :after_update_action
    # METODES DE CREATION DES DOCUENTS 5e4a5ad322ec7927f7dc62eb 5e4a5ad322ec7927f7dc62ea
    def compute_required_documents
      self.required_document_ids = []
      find_or_create_documents
    end
    
    def get_required_documents_labels
      labels = []
      required_documents.each do |doc|
        
        labels.push(id: doc.id,
                    type: doc.type,
                    applicant_id: doc.applicant_id,
                    status: doc.status,
                    rf_nud: (doc.folder_where_is_required_for_nud.include? id),
                    rf_consultation: (doc.folder_where_is_required_for_consultation.include? id),
                    rf_appli: (doc.folder_where_is_required_for_application.include? id))
      end
      labels
    end
    
    def all_applicant_ids
      all_applicant_ids = []
      [housing_request.applicant_ids, housing_request.main_applicant_id].flatten.uniq.compact.each do |id|
        all_applicant_ids.push id.to_s
      end
      all_applicant_ids
    end
    
    def find_or_create_documents
      compute_identity_documents(housing_request.main_applicant_id.to_s, true)
      if (!housing_request.is_untensed_area)
        housing_request.applicant_ids&.each do |applicant_id|
          compute_identity_documents(applicant_id.to_s, false)
        end
        
        
        # required_documents_common_type_list = ['hermes_identite', 'hermes_revenu_fiscal', 'hermes_ressources_mensuelles']
        all_applicant_ids.each do |applicant_id|
          
          compute_ressources_documents(applicant_id, applicant_id == housing_request.main_applicant_id.to_s)
          compute_revenus_fiscaux(applicant_id, applicant_id == housing_request.main_applicant_id.to_s)
          compute_dependents_documents(applicant_id, applicant_id == housing_request.main_applicant_id.to_s)
          compute_family_situation_documents(applicant_id, applicant_id == housing_request.main_applicant_id.to_s)
        end
        #   compute_others_documents_for_main_applicant(housing_request.main_applicant, true)
        compute_actual_housing_documents(housing_request.main_applicant_id.to_s, true)
        compute_motivation_documents(housing_request.main_applicant_id.to_s, true)
        compute_disabilities_documents(housing_request.main_applicant_id.to_s, true)
      
      end
    
    end
    
    def compute_identity_documents(applicant_id, is_main_applicant)
      applicant = Applicant.find(applicant_id)
      if applicant
        doc = case applicant.nationality
              when '1'
                {type: "Carte nationale d’identité ou passeport",
                 message: "Veuillez déposer une copie recto-verso de votre carte d'identité ou en cours de validité. 
Attention, ce document doit impérativement faire figurer 
Nom d’usage, Nom de naissance, Prénom (uniquement le premier),Date de naissance,Lieu de naissance,Pays de naissance, Date d’émission, Date d’expiration, Signature et photo. 
Ce document doit être lisible et complet.",
                 code: is_main_applicant ? 'CID' : 'CICO',
                 category: is_main_applicant ? 'IDD' : 'IDCO',
                 label: is_main_applicant ? "Carte d'identité ou passeport du demandeur" : "Carte d'identité ou passeport du conjoint ou du co-titulaire"}
              when '3'
                {type: "titre de séjour et Récépissé de renouvellement de titre de séjour si celui ci est expiré",
                 message: "Veuillez déposer une copie de votre titre de séjour en cours de validité. 
Attention, ce document doit impérativement faire figurer 
Civilité, Nom d’usage, Nom de naissance, Prénom (uniquement le premier),Date de naissance,Lieu de naissance,Pays de naissance, Date d’émission, Date d’expiration, Signature et photo.
Ce document doit être lisible et complet. 
Si le titre de séjour est expiré, veuillez déposer le récépissé du renouvellement de votre titre de séjour datant de moins de 6 mois",
                 code: is_main_applicant ? 'TSD' : 'TSCO',
                 category: is_main_applicant ? 'IDD' : 'IDCO',
                 label: is_main_applicant ? 'Titre de séjour du demandeur' : 'Titre de séjour du conjoint ou du co-titulaire'}
              when '2'
                {type: "Carte d'identité d'un pays européen",
                 message: "Veuillez déposer une copie recto-verso de votre carte d'identité ou passeport en cours de validité. 
Ce document doit être lisible et complet. ",
                 code: is_main_applicant ? 'CID' : 'CICO',
                 category: is_main_applicant ? 'IDD' : 'IDCO',
                 label: is_main_applicant ? "Carte d'identité ou passeport du demandeur" : "Carte d'identité ou passeport du conjoint ou du co-titulaire"}
              end
        
        if doc
          
          find_or_create_required_document("Pièce d'identité", doc[:type], doc[:message], applicant.id.to_s, is_main_applicant, is_main_applicant, true, 'SPI', doc[:code], doc[:category], nil, doc[:label], true)
        end
      end
    end
    
    def compute_ressources_documents(applicant_id, _is_main_applicant)
      applicant = Applicant.find(applicant_id)
      return unless applicant
      
      rules = [
        {
          monthly_incomes: %w[AUT],
          #todo ===>souci
          required_documents: [{type: 'Justificatif(s) autres ressources perçues', message: 'Justificatif(s) autres ressources perçues', code: 'JUSR', category: 'RESS', label: 'Justificatif de ressources'}],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[BE],
          required_documents: [{
                                 type: "Carte d'étudiant délivrée par un établissement certifié et valable pour l'année en cours",
                                 message: "Veuillez déposer ici une copie recto verso de votre carte d'étudiant ou une attestation scolaire pour l'année en cours établie par un établissement certifié.Ce document doit être lisible et complet. ",
                                 code: 'CAE',
                                 category: 'SITPRO',
                                 label: "Carte d'étudiant"
                               },
                               {
                                 type: "Avis d'attribution de bourse",
                                 message: "Veuillez déposer ici une copie de votre avis d'attribution de bourse CROUS. Attention, ce document doit impérativement faire figurer votre nom, prénom, ainsi que les montants perçus. Ce document doit être lisible et complet. "
                               }],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[RET],
          required_documents: [{type: "Notification de pension",
                                message: "Veuillez déposer ici une copie de votre attestation de retraite. 
Vous pouvez la retrouver sur https://www.lassuranceretraite.fr/portail-info/home.html. 
Attention, ce document doit impérativement faire figurer votre régime de retraite et les montants perçus. 
Ce document doit être lisible et complet.",
                                code: 'JUSR',
                                category: 'RESS',
                                label: 'Justificatif de ressources'}],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[PINV],
          #todo===> a verifier
          required_documents: [{type: 'Attestation de la Caisse d’allocations familiales (CAF) /Mutualité sociale agricole (MSA)',
                                message: "Veuillez déposer ici une copie de l'attestation de la CAF OU de la MSA si vous faites partie du secteur agricole. 
Attention, ce document doit impérativement faire figurer nom, prénom et montants perçus. 
Ce document doit être lisible et complet. ",
                                code: 'JUSR',
                                category: 'RESS',
                                label: 'Justificatif de ressources'}],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[APP],
          required_documents: [{type: "Contrat de travail de l'année en cours/ Bulletins de salaire",
                                message: "Veuillez déposer ici vos 3 derniers bulletins de salaire ou votre contrat d'apprentissage/ professionnalisation pour l'année en cours. Ce document doit être lisible et complet. ",
                                code: 'CONT',
                                category: 'SITPRO',
                                label: 'Contrat de travail'}],
          r_for_consultation: _is_main_applicant,
          r_for_application: true,
          kyc: true
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[AUT],
          #todo===>souci=
          required_documents: [{type: 'Toute pièce établissant la situation indiquée', message: 'Toute pièce établissant la situation indiquée', code: 'ATTF', category: 'SITPRO', label: 'Attestation de formation ou autre justificatif de situation'}],
          r_for_consultation: _is_main_applicant,
          r_for_application: true
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[CDI CDD CDIP AGP],
          required_documents: [{
                                 type: "Bulletin de salaire privé / public  ou Attestation de l'employeur",
                                 message: "Veuillez déposer ici vos 3 derniers bulletins de salaire. 
Attention, ces documents doivent impérativement faire figurer  Nom d’usage, Nom de naissance, Prénom, SIRET, Salaire net mensuel perçu, Date du document, Prélèvement à la source si simple ?
Ce document doit être lisible et complet. 
ou
Veuillez déposer ici une attestation de votre employeur datant de moins de 3 mois. Ce document doit être lisible et complet.",
                                 code: 'FPD',
                                 category: 'RESS',
                                 label: 'Fiche de paie du demandeur et le cas échéant de son conjoint et de chaque co-titulaire'}],
          r_for_consultation: _is_main_applicant,
          r_for_application: true,
          kyc: true
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[ART],
          required_documents: [{type: "Dernier bilan ou attestation du comptable de l'entreprise évaluant le salaire mensuel perçu ou tout document comptable habituellement fourni à l'administration permettant de justifier les revenus",
                                message: "Veuillez déposer ici un bilan ou une attestation comptable de l'entreprise ou tout document vous permettant de justifier de vos revenus datant de moins de 3 mois.
Ce document doit être lisible et complet.",
                                code: 'JUSR', category: 'RESS', label: 'Justificatif de ressources'}],
          r_for_consultation: _is_main_applicant,
          r_for_application: true,
          kyc: true
        }, {
          monthly_incomes: %w[AMV],
          #todo===>pas trouvé
          required_documents: [{type: "Notification de pension de minimum vieillesse", message: 'Notification de pension de minimum vieillesse', code: 'JUSR', category: 'RESS', label: 'Justificatif de ressources'}],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[PAR],
          required_documents: [{type: "Extrait du jugement de divorce ou autre document démontrant la perception de la pension",
                                message: "Veuillez déposer ici une copie de l'extrait du jugement attestant du montant perçu ou, dans le cas d'un divorce à l'amiable, une attestation sur l'honneur. 
Ce document doit être lisible et complet.",
                                code: 'JUSR',
                                category: 'RESS',
                                label: 'Justificatif de ressources'}],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[CHO],
          required_documents: [{type: "Avis de paiement de l'ARE",
                                message: 'Veuillez déposer ici une copie de votre avis de paiement Pôle Emploi en cours de validité. 
Vous pouvez retrouver ce document sur votre espace personnel Pôle Emploi. 
Attention, ce document doit impérativement faire figurer les montants perçus. 
Ce document doit être lisible et complet. ',
                                code: 'JUSR',
                                category: 'RESS',
                                label: 'Justificatif de ressources'}],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[
            AF
            AAH
            AEEH
            AJPP
            RSA
            PAJE
            PDA
          ],
          required_documents: [{type: "Attestation de la Caisse d’allocations familiales (CAF) /Mutualité sociale agricole (MSA)",
                                message: "Veuillez déposer ici une copie de l'attestation de la CAF OU de la MSA si vous faites partie du secteur agricole. 
Attention, ce document doit impérativement faire figurer nom, prénom et montants perçus. 
Ce document doit être lisible et complet. ",
                                code: 'CAF',
                                category: 'REV',
                                label: "Certificat de l'administration fiscale"}],
          r_for_consultation: false,
          r_for_application: true
        }
      ]
      if applicant&.monthly_charges && ((applicant&.monthly_charges["PAV"]&.to_i ||
        applicant.monthly_charges["PAV".to_sym]&.to_i ||
        0
      ) > 0)
        find_or_create_required_document(
          "Ressources",
          "Extrait de jugement ou autre document démontrant le versement de la pension",
          "Veuillez déposer ici une copie de l'extrait du jugement attestant du montant versé ou, dans le cas d'un divorce à l'amiable, une attestation sur l'honneur. Ce document doit être lisible et complet. ",
          applicant.id.to_s,
          false,
          false,
          true,
          'SPR',
          "JUSR",
          "RESS",
          nil,
          'Justificatif de ressources',
          false)
      end
      rules.each do |rule|
        work_contract_type_condition = true
        if rule[:work_contract_type]
          work_contract_type_condition = rule[:work_contract_type].include? applicant.work_contract_type
        end
        is_doc_needed = false
        rule[:monthly_incomes].each do |ressource_label|
          next unless applicant.monthly_incomes &&
            (
            applicant.monthly_incomes[ressource_label]&.to_i ||
              applicant.monthly_incomes[ressource_label.to_sym]&.to_i ||
              0
            ) > 0 && work_contract_type_condition
          
          is_doc_needed = true
        end
        next unless is_doc_needed
        
        
        rule[:required_documents].each do |req_doc|
          find_or_create_required_document("Ressources", req_doc[:type], req_doc[:message], applicant.id.to_s, false, rule[:r_for_consultation], rule[:r_for_application], 'SPR', req_doc[:code], req_doc[:category], nil, req_doc[:label], req_doc[:kyc])
        end
      end
    end
    
    def compute_others_documents_for_main_applicant(applicant_id, _is_main_applicant)
      applicant = Applicant.find(applicant_id)
      return unless applicant
      
      if applicant.family_situation == 'V'
        find_or_create_required_document(
          #todo===> veuf pas trouvé
          'Certificat de décès ou livret de famille',
          'Certificat de décès ou livret de famille',
          'Certificat de décès ou livret de famille',
          applicant.id.to_s,
          false,
          true,
          true,
          'SPR'
        )
      end
    end
    
    def compute_revenus_fiscaux(applicant_id, is_main_applicant)
      applicant = Applicant.find(applicant_id)
      return unless applicant
      
      if applicant&.amount_year_minus_two && applicant&.amount_year_minus_two >= 0
        find_or_create_required_document(
          "Avis d'imposition",
          "Avis d'imposition",
          "Copie intégrale (4 pages) de l'Avis d'imposition ou de non imposition  de l'année N-2 ou avis de situation déclarative à l'impôt sur le revenu 
ou 
Copie intégrale de l'avis d'imposition ou de non imposition des parents si fiscalement rattaché.
ou
Imposition à l'étranger :
Avis d'imposition sur le revenu du pays ou attestation d'une administration compétente (Ces documents doivent être traduits en français et convertis en euros)
",
          applicant.id.to_s,
          false,
          true,
          true,
          'RFR',
          is_main_applicant ? 'AVID' : 'AVIDCH',
          'REV',
          false,
          is_main_applicant ? "Avis d'imposition ou de non imposition (année N-2 et le cas échéant N-1) du demandeur et le cas
            échéant de son conjoint" : "Avis d'imposition ou de non imposition (année N-2 et le cas échéant N-1) du co-titulaire ou des
            personnes à charge",
          true
        )
      end
      if applicant&.amount_year_minus_one && applicant&.amount_year_minus_one >= 0
        find_or_create_required_document(
          "Avis d'imposition",
          "Avis d'imposition",
          "Copie intégrale (4 pages) de l'Avis d'imposition ou de non imposition  de l'année N-1 ou avis de situation déclarative à l'impôt sur le revenu 
ou 
Copie intégrale de l'avis d'imposition ou de non imposition des parents si fiscalement rattaché.
ou
Imposition à l'étranger :
Avis d'imposition sur le revenu du pays ou attestation d'une administration compétente (Ces documents doivent être traduits en français et convertis en euros)
",
          applicant.id.to_s,
          false,
          true,
          true,
          'RFR',
          is_main_applicant ? 'AVID' : 'AVIDCH',
          'REV',
          false,
          is_main_applicant ? "Avis d'imposition ou de non imposition (année N-2 et le cas échéant N-1) du demandeur et le cas
            échéant de son conjoint" : "Avis d'imposition ou de non imposition (année N-2 et le cas échéant N-1) du co-titulaire ou des
            personnes à charge",
          true
        )
      end
    end
    
    def compute_family_situation_documents(applicant_id, is_main_applicant)
      applicant = Applicant.find(applicant_id)
      return unless applicant
      
      rules = [
        {
          situation: 'P',
          required_document: {type: 'Attestation d\'enregistrement de PACS',
                              message: "Veuillez déposer ici une copie du récépissé de l'enregistrement du PACS délivré par votre mairie/
 un duplicata du PACS si celui a été enregistré chez un notaire/ 
une copie de vos actes de naissance respectifs où il est fait mention de votre Pacs. 
Ce document doit être lisible et complet."},
          not_for_coap: true
        }, {
          situation: 'M',
          required_document: {type: 'Livret de famille ou tout document équivalant démontrant le mariage',
                              message: "Veuillez déposer ici l'extrait du livret de famille ou tout document équivalent démontrant le mariage. Attention, les noms des deux conjoints doivent être visibles. Ce document doit être lisible et complet. ",
                              code: is_main_applicant ? nil : 'LFCO',
                              category: is_main_applicant ? nil : 'IDCO',
                              label: is_main_applicant ? nil : "Livret de famille ou acte d'État civil"},
          not_for_coap: true
        }, {
          situation: 'D',
          required_document: {type: "Jugement de divorce ou convention homologuée en cas de divorce par consentement mutuel",
                              message: 'Veuillez déposer ici une copie du jugement de divorce OU la convention homologuée si vous avez divorcé par consentement mutuel. 
Ce document doit être lisible et complet.',
                              code: 'JUGFAM',
                              category: 'MOTIF',
                              label: 'Jugement de divorce, 
ordonnance de non conciliation ou autres jugements familiaux'}
        }, {
          situation: 'S',
          required_document: {type: "Jugement de divorce ou convention homologuée en cas de divorce par consentement mutuel 
ou
 Mention de la dissolution dans l’acte de naissance
ou
Ordonnance de non-conciliation ou, à défaut, copie de l’acte de saisine du juge aux affaires familiales dans les conditions prévues au code de procédure civile ou, lorsque c’est un divorce par consentement mutuel, justificatif de l’avocat du demandeur
ou
Attestation de dépôt de demande de regroupement familial",
                              message: "(Vous êtiez marié(e))
« Veuillez déposer ici une copie du jugement de divorce OU la convention homologuée si vous avez divorcé par consentement mutuel. Ce document doit être lisible et complet.  »
ou
(Vous êtiez pacsé(e))
« Veuillez déposer ici une copie de la mention de dissolution du PACS figurant sur votre acte de naissance. Ce document doit être lisible et complet. »
ou
(Autre)
« Récépissé du dépôt de plainte si violence au sein du couple »
"}
        }, {
          situation: 'V',
          #todo====>souci
          required_document: {type: "Certificat de décès ou livret de famille",
                              message: 'Certificat de décès ou livret de famille'}
        }
      
      ]
      
      rules.each do |rule|
        if rule[:situation] == applicant.family_situation
          (is_main_applicant && rule[:not_for_coap]) || !rule[:not_for_coap]
          find_or_create_required_document("Situation familiale", rule[:required_document][:type], rule[:required_document][:message], applicant.id.to_s, false, false, true, 'SF', rule[:required_document][:code], rule[:required_document][:category], false, rule[:required_document][:label])
        end
      end
    end
    
    def compute_actual_housing_documents(applicant_id, is_main_applicant)
      applicant = Applicant.find(applicant_id)
      return unless applicant
      
      rules = [
        {
          actual_housing_detail: %w[POCC],
          #todo =>>>attention, piece réunifiées redédoublées
          required_documents: [{
                                 type: "Acte de propriété",
                                 message: 'Veuillez déposer ici une copie de votre acte de propriété. Ce document doit être lisible et complet. ',
                                 code: 'COLOC',
                                 category: 'LOGACT',
                                 label: 'Contrat de location / justificatif de propriété'},
                               {
                                 type: "Plan de financement",
                                 message: 'Veuillez déposer ici une copie de votre plan de financement en cours de validité. Ce document doit être lisible et complet. '
                               }]
        }, {
          actual_housing_detail: %w[RSLL RHVS HSH HCDFCM],
          required_documents: [{type: "Attestation de la structure d’hébergement ou du gestionnaire du logement-foyer",
                                message: "Veuillez déposer ici une attestation de votre structure d'hébergement ou produite par le gestionnaire de votre logement-foyer datant de moins de 3 mois. 
Attention votre nom, celui de l'hébergeant ainsi que l'adresse doivent impérativement figurer sur ce document. 
Ce document doit être lisible et complet. "}]
        }, {
          #todo ===>absent de la nouvelle liste
          actual_housing_detail: %w[FONC],
          required_documents: [{type: "Contrat de bail ou dernière quittance de loyer ou tout document attestant de la situation", message: 'Contrat de bail ou dernière quittance de loyer ou tout document attestant de la situation',
                                code: 'JUSTL',
                                category: 'LOGACT',
                                label: 'Quittance ou tout autre justificatif de loyer à jour'}]
        }, {
          #todo ===>absent de la nouvelle liste
          actual_housing_detail: %w[HCPE HCP],
          required_documents: [{type: 'Attestation de la personne qui héberge',
                                message: "Veuillez déposer ici une attestation datant de moins de 3 mois rédigée par la personne qui vous héberge. 
Attention votre nom, celui de l'hébergeant ainsi que l'adresse doivent impérativement figurer sur ce document.
Ce document doit être lisible et complet. ",
                                code: 'ATTH',
                                category: 'LOGACT',
                                label: "Attestation d'hébergement ou de domiciliation"}]
        }, {
          actual_housing_detail: %w[SD CAMCAR HHOT HABMOB],
          required_documents: [{
                                 type: "Attestation d’un travailleur social, d’une association ou certificat de domiciliation",
                                 message: "Veuillez déposer ici une copie de l'attestation de domiciliation du travailleur social/association suivant votre dossier datant de moins de 3 mois.
Ce document doit être lisible et complet."
                               }]
        }, {
          actual_housing_detail: %w[HLM LP],
          required_documents: [{type: "Contrat de bail et quittance ou, à défaut de quittance, attestation du bailleur indiquant que vous êtes à jour de vos loyers et charges, ou tout moyen de preuve des paiements effectués",
                                message: "Veuillez déposer ici une copie de votre dernière quittance de loyer OU une attestation de votre bailleur datant de moins de 3 mois indiquant que votre situation locative est à jour. 
Attention, votre nom doit impérativement figurer sur ce document. 
Ce document doit être lisible et complet."}]
        }
      
      ]
      
      rules.each do |rule|
        next unless housing_request.housing_type_list.is_a?(Array) && (rule[:actual_housing_detail] & housing_request.housing_type_list).length > 0
        
        rule[:required_documents].each do |req_doc|
          find_or_create_required_document("Logement Actuel",
                                           req_doc[:type],
                                           req_doc[:message],
                                           applicant.id.to_s,
                                           false, false,
                                           true, 'SPR', req_doc[:code],
                                           req_doc[:category],
                                           false,
                                           req_doc[:label])
        end
      end
      
      if is_main_applicant && housing_request.housing_type_list && housing_request.housing_type_list.include?('SLHT')
        #todo => sous locataire=>cas absent
        find_or_create_required_document("Logement Actuel",
                                         "(Vous êtes sous locataire)
(« Bail et dernière quittance de loyer ou attestation du bailleur indiquant que le locataire est à jour des loyers et charges ou tout moyen de preuve des paiements effectués. »

(Vous êtes hébergé dans un logement à titre temporaire)
« Attestation de la personne qui héberge »
", "(Vous êtes sous locataire)
(« Bail et dernière quittance de loyer ou attestation du bailleur indiquant que le locataire est à jour des loyers et charges ou tout moyen de preuve des paiements effectués. »

(Vous êtes hébergé dans un logement à titre temporaire)
« Attestation de la personne qui héberge »
",
                                         applicant.id.to_s,
                                         false,
                                         false,
                                         true,
                                         'SPL')
      end
    end
    
    def compute_motivation_documents(applicant_id, _is_main_applicant)
      all_motivations = (1..3).map {|i| housing_request.send("motivation#{i}")}
      rules = [
        {
          motivation: '08',
          required_document: {type: "Certificat médical",
                              message: "Veuillez déposer ici une copie du certificat médical établi à votre nom. Ce document doit être lisible et complet. ",
                              code: 'JUSBA',
                              category: 'COMPH',
                              label: "Justificatif des besoins d'adaptation (certificat médical ou autre document)"}
        }, {
          motivation: '28',
          #todo: absent
          required_document: {
            type: 'Attestation de l\'employeur actuel ou futur justifiant votre mobilité professionnelle',
            message: 'Attestation de l\'employeur actuel ou futur justifiant votre mobilité professionnelle'}
        }, {
          motivation: '15',
          required_document: {type: "Attestation de dépôt de demande de regroupement familial",
                              message: "Veuillez déposer ici une copie de l'attestation de dépôt de demande de regroupement familial. 
Ce document doit être lisible et complet. "}
        }, {
          motivation: '02',
          #todo: absent
          required_document: {type: 'Toute pièce établissant de la démolition de votre logement actuel', message: 'Toute pièce établissant de la démolition de votre logement actuel'}
        }, {
          motivation: '07',
          required_document: {type: "Carte d’invalidité ou décision d’une commission administrative compétente (commissions des droits et de l’autonomie des personnes handicapées) ou d’un organisme de sécurité sociale",
                              message: "Veuillez déposer ici une copie de 
votre carte d'invalidité 
OU de la décision d’une commission administrative compétente (Commissions des droits et de l'autonomie des personnes handicapées) 
OU d’un organisme de sécurité sociale. Attention, le document doit impérativement faire figurer vos nom et prénom.
Ce document doit être lisible et complet. ",
                              code: 'CARINV',
                              category: 'COMPH',
                              label: "Carte d'invalidité ou décision de commission administrative compétente (MDPH...)"}
        }, {
          motivation: '05',
          required_document: {type: "Commandement de payer ou assignation à comparaître ou jugement prononçant l’expulsion ou commandement de quitter les lieux",
                              message: "Veuillez déposer ici une copie du commandement de payer OU de l'assignation à comparaître OU du jugement prononçant l'expulsion ou demande de quitter les lieux. 
Attention, le document doit impérativement faire figurer votre nom, prénom, adresse. 
Ce document doit être lisible et complet. ",
                              code: 'JUGEXP',
                              category: 'MOTF',
                              label: "Jugement d'expulsion"}
        }, {
          motivation: '22',
          #todo=> absent
          required_document: {type: 'Plan d\'apurement de la dette ou démarches en cours attestées par un travailleur social ou une association ou autre document démontrant les difficultés',
                              message: 'Plan d\'apurement de la dette ou démarches en cours attestées par un travailleur social ou une association ou autre document démontrant les difficultés',
                              code: 'LETI',
                              category: 'MOTIF',
                              label: "Lettre de l'intéressé ou d'un intervenant"}
        }, {
          motivation: '26',
          required_document: {type: "Agrément assistant(e) maternel(le)",
                              message: "Veuillez déposer ici une copie de votre agrément de profession. Attention, ce document doit impérativement faire figurer vos nom, prénom et adresse. Ce document doit être lisible et complet. "}
        }, {
          motivation: '06',
          required_document: {type: "Ordonnance de non-conciliation ou, à défaut, copie de l’acte de saisine du juge aux affaires familiales dans les conditions prévues au code de procédure civile ou, lorsque c’est un divorce par consentement mutuel, justificatif de l’avocat du demandeur",
                              message: "Veuillez déposer ici une copie de l'ordonnance de non-conciliation OU une copie de l’acte de saisine du juge aux affaires familiales dans les conditions prévues au code de procédure civile ou, lorsque c’est un divorce par consentement mutuel, le justificatif de l’avocat du demandeur. Ce document doit être lisible et complet."
          }
        }, {
          motivation: '04',
          required_document: {type: "Lettre de congé du propriétaire ou jugement prononçant la résiliation du bail",
                              message: "Veuillez déposer ici une copie de la lettre de congé du propriétaire du logement OU le jugement prononçant la résiliation du bail de votre logement actuel. Ce document doit être lisible et complet.",
                              code: 'CONPR',
                              category: 'MOTIF',
                              label: 'Lettre de congé du propriétaire'}
        }, {
          motivation: '24',
          #todo=> absent
          required_document: {type: "Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement ou copie du jugement d'un tribunal statuant sur l'indécence du logement ou attestation de la CAF/MSA ou autre document démontrant l'indécence du logement", message: "Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement ou copie du jugement d'un tribunal statuant sur l'indécence du logement ou attestation de la CAF/MSA ou autre document démontrant l'indécence du logement"}
        }, {
          motivation: '25',
          required_document: {type: "Logement insalubre ou dangereux :
Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement, photos, jugement du tribunal, attestation de la CAF ou de la CMSA, copie d'une décision de l'administration (mise en demeure, arrêté préfectoral ou municipal d'insalubrité remédiable ou irrémédiable ou de péril, ou de fermeture administrative s'il s'agit d'un hôtel meublé) ou autre document démontrant l'insalubrité ou la dangerosité du logement ou de l'immeuble
Logement impropre à l'habitation :
Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement, photos, jugement du tribunal, arrêté préfectoral de mise en demeure de faire cesser l'occupation des lieux ou autre document démontrant le caractère impropre à l'habitation", message: "Logement insalubre ou dangereux :
Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement, photos, jugement du tribunal, attestation de la CAF ou de la CMSA, copie d'une décision de l'administration (mise en demeure, arrêté préfectoral ou municipal d'insalubrité remédiable ou irrémédiable ou de péril, ou de fermeture administrative s'il s'agit d'un hôtel meublé) ou autre document démontrant l'insalubrité ou la dangerosité du logement ou de l'immeuble
Logement impropre à l'habitation :
Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement, photos, jugement du tribunal, arrêté préfectoral de mise en demeure de faire cesser l'occupation des lieux ou autre document démontrant le caractère impropre à l'habitation"}
        }
      
      ]
      
      rules.each do |rule|
        if all_motivations.include?(rule[:motivation])
          find_or_create_required_document("Motifs de la demande",
                                           rule[:required_document][:type],
                                           rule[:required_document][:message],
                                           applicant_id,
                                           false,
                                           false,
                                           true,
                                           'MD',
                                           rule[:required_document][:code],
                                           rule[:required_document][:category],
                                           false,
                                           rule[:required_document][:label])
        end
      end
    end
    
    def compute_dependents_documents(applicant_id, is_main_applicant)
      if is_main_applicant
        if housing_request.unborn_children&.length && housing_request.unborn_children[0] && housing_request.unborn_children[0]['anticipated_birth'] &&
          Date.parse(housing_request.unborn_children[0]['anticipated_birth']) - 196.days < Date.today &&
          Date.today < Date.parse(housing_request.unborn_children[0]['anticipated_birth'])
          find_or_create_required_document(
            "Dépendants",
            'Certificat de grossesse attestant que la grossesse est supérieure à 12 semaines',
            "Veuillez déposer ici une copie du certificat de grossesse datant de moins de 6 mois. Cette attestation peut vous être délivrée par votre médecin traitant. Ce document doit être lisible et complet.",
            applicant_id.to_s,
            false,
            false,
            true,
            'PC',
            'CGCH',
            'PCH',
            false,
            'Certificat de grossesse'
          )
        end
        
        if housing_request&.dependents && housing_request&.dependents&.length > 0
          housing_request.dependents.each do |dep|
            dep_infos = dep.to_json
            if (dep["rel_type"] == "E" || dep["coparentality"] == "G" || dep["coparentality"] == "D")
              if (dep["birth_date"] && (Date.parse(dep["birth_date"]) <= Date.today - 18.years)) #todo
                find_or_create_required_document(
                  "Dépendants",
                  dep_infos + "####Carte nationale d'identité ou passeport ou titre de séjour ainsi que le récépissé de renouvellement de titre de séjour si celui-ci est expiré ",
                  "Carte nationale d'identité ou passeport ou titre de séjour ainsi que le récépissé de renouvellement de titre de séjour si celui-ci est expiré",
                  applicant_id.to_s,
                  false,
                  false,
                  true,
                  'PC',
                  'CICH',
                  'PCH',
                  true,
                  "Carte d'identité ou passeport d'une personne à charge"
                )
              end
              find_or_create_required_document(
                "Dépendants",
                dep_infos + '####Livret de famille ou acte de naissance',
                "Veuillez déposer ici une copie de votre livret de famille ou acte de naissance de tous les enfants devant vivre au foyer mentionnant votre nom. Ce document doit être lisible et complet.",
                applicant_id.to_s,
                false,
                false,
                true,
                'PC',
                'LFCH',
                'PCH',
                true,
                "Livret de famille ou acte d'État civil"
              )
            end
            next unless dep['rel_type'] == 'P' || dep['rel_type'] == 'A'
            
            find_or_create_required_document(
              "Dépendants",
              dep_infos + "####Carte nationale d'identité ou passeport ou titre de séjour ainsi que le récépissé de renouvellement de titre de séjour si celui-ci est expiré ",
              "Carte nationale d'identité ou passeport ou titre de séjour ainsi que le récépissé de renouvellement de titre de séjour si celui-ci est expiré ",
              applicant_id.to_s,
              false,
              false,
              true,
              'PC',
              'CICH',
              'PCH',
              true,
              "Carte d'identité ou passeport d'une personne à charge"
            )
          end
        end
      end
    end
    
    def compute_disabilities_documents(applicant_id, is_main_applicant)
      if housing_request&.disability_details && housing_request&.disability_details.is_a?(Array) && housing_request&.disability_details&.length > 0
        housing_request.disability_details.each_with_index do |dis, i|
          find_or_create_required_document(
          "handicap",
            '{"label":"Pour la personne handicapée n°' + (i + 1).to_s + '"}' + "****« Carte d'invalidité » 
Ou
 « Décision d'une commission administrative compétente (commission départementale de l'éducation spéciale, commission technique d'orientation et de reclassement professionnel, commissions des droits et de l'autonomie des personnes handicapées) ou d'un organisme de sécurité sociale »
Ou
 « Carte « mobilité inclusion » portant la mention « invalidité » au sens de l’article L.241-3 du code de l’action sociale et des familles ».
",
          '{"label":"Pour la personne handicapée n°' + (i + 1).to_s + '"}' + "****« Carte d'invalidité » 
Ou
 « Décision d'une commission administrative compétente (commission départementale de l'éducation spéciale, commission technique d'orientation et de reclassement professionnel, commissions des droits et de l'autonomie des personnes handicapées) ou d'un organisme de sécurité sociale »
Ou
 « Carte « mobilité inclusion » portant la mention « invalidité » au sens de l’article L.241-3 du code de l’action sociale et des familles ».
",
            applicant_id.to_s,
            false,
            false,
            true,
            'HANDI',
            "CARINV",
            "COMPH",
            nil,
            nil
          )
        end
      end
    end
    
    # methode permettant d'associer au housing_request_folder un requiredDocument
    def find_or_create_required_document(nature, type, label, applicant_id, is_required_for_nud, is_required_for_consultation, is_required_for_application, category, sne_code = nil, sne_category = nil, same_for_all_applicant = false, sne_piece_label = nil, kyc_needed = false)
      required_document = if same_for_all_applicant
                            Hermes::RequiredDocument.find_by(
                              type: type,
                              applicant_id: {'$in' => all_applicant_ids}
                            )
                          else
                            Hermes::RequiredDocument.find_by(
                              type: type,
                              applicant_id: applicant_id.to_s
                            )
                          end
      
      if required_document
        if is_required_for_nud
          unless required_document.folder_where_is_required_for_nud
            required_document.folder_where_is_required_for_nud = [id]
          end
          unless required_document.folder_where_is_required_for_nud.include?(id)
            required_document.folder_where_is_required_for_nud.push(id)
          end
        end
        if is_required_for_consultation
          unless required_document.folder_where_is_required_for_consultation
            required_document.folder_where_is_required_for_consultation = [id]
          end
          unless required_document.folder_where_is_required_for_consultation.include?(id)
            required_document.folder_where_is_required_for_consultation.push(id)
          end
        end
        if is_required_for_application
          unless required_document.folder_where_is_required_for_application
            required_document.folder_where_is_required_for_application = [id]
          end
          unless required_document.folder_where_is_required_for_application.include?(id)
            required_document.folder_where_is_required_for_application.push(id)
          end
        end
        required_document.save # TODO: OPTIMISATION ne pas save si pas de changement
      end
      required_document_ids.push(
        if !required_document
          Hermes::RequiredDocument.create(
            nature: nature,
            type: type,
            label: label,
            applicant_id: applicant_id.to_s,
            status: 'Pièce manquante',
            folder_where_is_required_for_nud: is_required_for_nud ? [id] : [],
            folder_where_is_required_for_consultation: is_required_for_consultation ? [id] : [],
            folder_where_is_required_for_application: is_required_for_application ? [id] : [],
            sne_code: sne_code,
            sne_category: sne_category,
            sne_piece_label: sne_piece_label,
            category: category,
            user_id: user_id,
            kyc_needed: kyc_needed
          ).id
        else
          required_document.id
        end
      )
    end
    
    def get_file_list(unique_number)
      so = Hermes::BoClient::ListingPJTask.new(
        housing_request_id: housing_request.id,
        num_unique: unique_number
      )
      validate_xml(so.to_xml, 'recherchePJ.xsd')
      ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml, "post", "Hermes::HousingRequestFolder", id.to_s)
    end
    
    def syncro_files_from_sne(unique_number)
      housing_request_unique_numbers = housing_request.nuds.only(:unique_number).pluck(:unique_number)
      unless unique_number && (housing_request_unique_numbers.include? unique_number)
        return
      end
      
      if Rails.env == 'test'
        path = [Hermes::Engine.root, %w[spec xml_bouchon], '33a.xml'].flatten.join('/').to_s
        pn = Pathname.new(path)
        file_list = File.read(pn)
      else
        file_list = get_file_list(unique_number)
      end
      
      if (file_list)
        
        parsed_xml = Hash.from_xml(file_list.gsub("\n", ''))
        
        piecesArray = (parsed_xml['recherche_PJ_Reponse']['informationPJ'].is_a? Array) ? parsed_xml['recherche_PJ_Reponse']['informationPJ'] : [parsed_xml['recherche_PJ_Reponse']['informationPJ']]
        required_documents.each do |required_doc|
          found = false
          need_to_upload = required_doc.sne_code && required_doc.sne_piece_label && required_doc.doc_id
          piecesArray.each do |existing_doc|
            unless !found && existing_doc['type'] == required_doc.sne_piece_label
              next
            end
            
            found = true
            need_to_download = !required_doc.doc_id || required_doc.last_user_send_file_date > DateTime.parse(existing_doc['dateDepot'])
            if required_doc.last_user_send_file_date && required_doc.last_user_send_file_date < Date.parse(existing_doc['dateDepot'])
              need_to_upload = false
            end
            if need_to_download
              required_doc.get_file_from_sne(existing_doc, unique_number)
            end
          end
          if need_to_upload
            if required_doc.exists_in_sne
              required_doc.update_file_in_sne(unique_number, housing_request_id)
            else
              required_doc.create_file_in_sne(unique_number, housing_request_id)
            end
          end
        end
        self.file_upload_from_sne_complete = true
      end
    end
    
    def status_for_required_document_list(required_document_list)
      incomplete = 0
      validation_errors = 0
      ready_for_treatment = 0
      validation_incomplete = 0
      validated = 0
      expired = 0
      
      required_document_list.each do |rd|
        case rd.status
        when 'Pièce manquante'
          incomplete += 1
        when 'Pièce invalide'
          validation_errors += 1
        when 'Contrôle en cours'
          validation_incomplete += 1
        when 'Prêt pour traitement'
          ready_for_treatment += 1
        when 'Pièce validée'
          if rd.kyc_needed
            if !rd.validity_limit || DateTime.current < rd.validity_limit
              validated += 1
            else
              expired += 1
            end
          else
            validated += 1
          end
        else
          incomplete += 1
        end
      end
      
      if incomplete > 0
        'Dossier incomplet'
      else
        if validation_errors > 0
          'Erreurs de validation dans les pieces fournies'
        else
          if ready_for_treatment > 0
            'Dossier complet prêt pour traitement'
          else
            if validation_incomplete > 0
              'Dossier complet en cours de traitement'
            else
              if expired > 0
                'Date de validité dépassée'
              else
                if validated == required_document_list.length
                  'Dossier validé'
                else
                  'Dossier incomplet'
                end
              end
            end
          end
        end
      end
    end
    
    def start_treatment
      doc_where_kyc_neded = []
      required_documents.each do |required_document|
        if required_document.kyc_needed && required_document.doc_id && required_document.status != 'Pièce validée'
          doc_where_kyc_neded.push(required_document)
        end
      end
      
      doc_where_kyc_neded.each do |doc_to_kyc|
        # TODO: enlever bouchon
        doc_to_kyc.status = 'Pièce validée'
        doc_to_kyc.validation_errors = '[]'
        doc_to_kyc.validity_limit = DateTime.current + 3.months
        doc_to_kyc.save
      end
    end
    
    def compute_statuses
      required_documents_for_nud = []
      required_documents_for_consultation = []
      required_documents_for_application = []
      required_documents.each do |required_document|
        required_document.reload
        if required_document.folder_where_is_required_for_nud.include? id
          required_documents_for_nud.push(required_document)
        end
        if required_document.folder_where_is_required_for_consultation.include? id
          required_documents_for_consultation.push(required_document)
        end
        if required_document.folder_where_is_required_for_application.include? id
          required_documents_for_application.push(required_document)
        end
      end
      
      self.nud_status = status_for_required_document_list(required_documents_for_nud) #+ "pour consultation."
      self.consultation_status = status_for_required_document_list(required_documents_for_consultation) #+ "pour consultation."
      self.application_status = status_for_required_document_list(required_documents_for_application) #+ "pour candidater."
    
    end
    
    def transfer_status_to_housing_request
      save_nedded = false
      if nud_status_changed? || consultation_status_changed? || application_status_changed?
        if application_status == "Dossier validé"
          housing_request.request_status = "COMPLETE"
          save_nedded = true
        else
          if consultation_status == "Dossier validé"
            housing_request.request_status = "ELIGIBLE"
            save_nedded = true
          else
            if (nud_status == 'Dossier validé')
              housing_request.request_status = "EN ATTENTE DE NUD"
              save_nedded = true
            else
              housing_request.request_status = "INCOMPLETE"
              save_nedded = true
            end
          end
        end
      end
      if (save_nedded == true && !housing_request.already_done && housing_request.request_status_changed?)
        housing_request.already_done = true
        housing_request.save
      end
    end
    
    def before_save_action
      compute_statuses if required_document_ids_changed?
    end
    
    def before_create_action
      self.user_id = housing_request&.user_id
    end
    
    def after_update_action
      
      transfer_status_to_housing_request if required_documents.length > 0
    end
  end
end
